@@ -5,12 +5,14 @@
  * @license GPL-2.0+
  */
 
+namespace LocalisationUpdate;
+
 /**
  * Fetches files over HTTP(s).
  */
-class LU_HttpFetcher implements LU_Fetcher {
+class HttpFetcher implements Fetcher {
 	public function fetchFile( $url ) {
-		return Http::get( $url );
+		return \Http::get( $url );
 	}
 
 	/**
@@ -18,11 +20,11 @@ class LU_HttpFetcher implements LU_Fetcher {
 	 * implementation of this.
 	 */
 	public function fetchDirectory( $pattern ) {
-		$files = array();
+		$files = [];
 
-		$languages = Language::fetchLanguageNames( null, 'mwfile' );
+		$languages = \Language::fetchLanguageNames( null, 'mwfile' );
 
-		foreach( array_keys( $languages ) as $code ) {
+		foreach ( array_keys( $languages ) as $code ) {
 			// Hack for core
 			if ( strpos( $pattern, 'Messages*.php' ) !== false ) {
 				$code = ucfirst( strtr( $code, '-', '_' ) );
