@@ -15,16 +15,9 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  * http://www.gnu.org/copyleft/gpl.html
  */
-
-/**
- * Stub object for maintaining backward-compatibility with extensions
- * that have not been updated for version 2.0 of SyntaxHighlight_GeSHi
- * and for supporting GeSHi lexer names whenever they map naturally to
- * a Pygments lexer.
- */
-class GeSHi {
+class SyntaxHighlightGeSHiCompat {
 	/** @var array A mapping of GeSHi lexer names to compatible Pygments lexers. */
-	public static $compatibleLexers = array(
+	private static $compatibleLexers = array(
 		// Assembler
 		'arm'       => 'asm',
 		'6502acme'  => 'asm',
@@ -114,19 +107,7 @@ class GeSHi {
 		'apt_sources' => 'debsources',
 	);
 
-	public function __construct( $html ) {
-		$this->html = $html;
-	}
-
-	public function error() {
-	}
-
-	public function set_language( $language ) {
-	}
-
-	public function parse_code() {
-		global $wgOut;
-		$wgOut->addModuleStyles( 'ext.pygments' );
-		return $this->html;
+	public static function getGeSHiToPygmentsMap() {
+		return self::$compatibleLexers;
 	}
 }
