@@ -47,17 +47,50 @@ ve.dm.MWSyntaxHighlightNode.static.toDataElement = function ( domElements, conve
 };
 
 ( function () {
-	var supportedLanguages = [ undefined ];
+	var supportedLanguages = [ undefined ],
+		geshiToPygmentsMap,
+		pygmentsToAceMap;
 
 	/**
-	 * Register supported languages.
+	 * Register supported Pygments languages.
 	 *
 	 * @param {Array} languages
 	 */
-	ve.dm.MWSyntaxHighlightNode.static.addLanguages = function ( languages ) {
+	ve.dm.MWSyntaxHighlightNode.static.addPygmentsLanguages = function ( languages ) {
 		ve.batchPush( supportedLanguages, languages );
 	};
 
+	/**
+	 * Register map from Geshi to pygments lexer names.
+	 *
+	 * @param {Array} map
+	 */
+	ve.dm.MWSyntaxHighlightNode.static.addGeshiToPygmentsMap = function ( map ) {
+		geshiToPygmentsMap = map;
+		ve.batchPush( supportedLanguages, Object.keys( geshiToPygmentsMap ) );
+	};
+
+	/**
+	 * Register a map from pygments to Ace lexer names.
+	 *
+	 * @param {Array} map
+	 */
+	ve.dm.MWSyntaxHighlightNode.static.addPygmentsToAceMap = function ( map ) {
+		pygmentsToAceMap = map;
+	};
+
+	/**
+	 * Converts a (valid) language as recognized by the SyntaxHighlight wikicode
+	 * to a compatible Ace lexer name (to be used by CodeEditor)
+	 *
+	 * @param {string} language Language name
+	 * @return {string} The name of the ace lexer
+	 */
+	ve.dm.MWSyntaxHighlightNode.static.convertLanguageToAce = function ( language ) {
+		language = geshiToPygmentsMap[ language ] || language;
+		return ( pygmentsToAceMap[ language ] || language ).toLowerCase();
+	};
+
 	/**
 	 * Check if a language is supported
 	 *
@@ -69,7 +102,7 @@ ve.dm.MWSyntaxHighlightNode.static.toDataElement = function ( domElements, conve
 	};
 
 	/**
-	 * Get an array of all languages
+	 * Get an array of all languages (both Pygments and former GeSHi names)
 	 *
 	 * @return {Array} All currently supported languages
 	 */
