@@ -43,16 +43,16 @@ class ApiFeedContributions extends ApiBase {
 
 		$config = $this->getConfig();
 		if ( !$config->get( 'Feed' ) ) {
-			$this->dieUsage( 'Syndication feeds are not available', 'feed-unavailable' );
+			$this->dieWithError( 'feed-unavailable' );
 		}
 
 		$feedClasses = $config->get( 'FeedClasses' );
 		if ( !isset( $feedClasses[$params['feedformat']] ) ) {
-			$this->dieUsage( 'Invalid subscription feed type', 'feed-invalid' );
+			$this->dieWithError( 'feed-invalid' );
 		}
 
 		if ( $params['showsizediff'] && $this->getConfig()->get( 'MiserMode' ) ) {
-			$this->dieUsage( 'Size difference is disabled in Miser Mode', 'sizediffdisabled' );
+			$this->dieWithError( 'apierror-sizediffdisabled' );
 		}
 
 		$msg = wfMessage( 'Contributions' )->inContentLanguage()->text();
@@ -79,6 +79,7 @@ class ApiFeedContributions extends ApiBase {
 			'deletedOnly' => $params['deletedonly'],
 			'topOnly' => $params['toponly'],
 			'newOnly' => $params['newonly'],
+			'hideMinor' => $params['hideminor'],
 			'showSizeDiff' => $params['showsizediff'],
 		] );
 
@@ -208,6 +209,7 @@ class ApiFeedContributions extends ApiBase {
 			'deletedonly' => false,
 			'toponly' => false,
 			'newonly' => false,
+			'hideminor' => false,
 			'showsizediff' => [
 				ApiBase::PARAM_DFLT => false,
 			],
