@@ -37,7 +37,7 @@
 class ApiQueryDisabled extends ApiQueryBase {
 
 	public function execute() {
-		$this->setWarning( "The \"{$this->getModuleName()}\" module has been disabled." );
+		$this->addWarning( [ 'apierror-moduledisabled', $this->getModuleName() ] );
 	}
 
 	public function getAllowedParams() {
