@@ -51,7 +51,7 @@ class ApiStashEdit extends ApiBase {
 		$params = $this->extractRequestParams();
 
 		if ( $user->isBot() ) { // sanity
-			$this->dieUsage( 'This interface is not supported for bots', 'botsnotsupported' );
+			$this->dieWithError( 'apierror-botsnotsupported' );
 		}
 
 		$cache = ObjectCache::getLocalClusterInstance();
@@ -61,9 +61,14 @@ class ApiStashEdit extends ApiBase {
 		if ( !ContentHandler::getForModelID( $params['contentmodel'] )
 			->isSupportedFormat( $params['contentformat'] )
 		) {
-			$this->dieUsage( 'Unsupported content model/format', 'badmodelformat' );
+			$this->dieWithError(
+				[ 'apierror-badformat-generic', $params['contentformat'], $params['contentmodel'] ],
+				'badmodelformat'
+			);
 		}
 
+		$this->requireAtLeastOneParameter( $params, 'stashedtexthash', 'text' );
+
 		$text = null;
 		$textHash = null;
 		if ( strlen( $params['stashedtexthash'] ) ) {
@@ -72,15 +77,18 @@ class ApiStashEdit extends ApiBase {
 			$textKey = $cache->makeKey( 'stashedit', 'text', $textHash );
 			$text = $cache->get( $textKey );
 			if ( !is_string( $text ) ) {
-				$this->dieUsage( 'No stashed text found with the given hash', 'missingtext' );
+				$this->dieWithError( 'apierror-stashedit-missingtext', 'missingtext' );
 			}
 		} elseif ( $params['text'] !== null ) {
 			// Trim and fix newlines so the key SHA1's match (see WebRequest::getText())
 			$text = rtrim( str_replace( "\r\n", "\n", $params['text'] ) );
 			$textHash = sha1( $text );
 		} else {
-			$this->dieUsage(
-				'The text or stashedtexthash parameter must be given', 'missingtextparam' );
+			$this->dieWithError( [
+				'apierror-missingparam-at-least-one-of',
+				Message::listParam( [ '<var>stashedtexthash</var>', '<var>text</var>' ] ),
+				2,
+			], 'missingparam' );
 		}
 
 		$textContent = ContentHandler::makeContent(
@@ -91,11 +99,11 @@ class ApiStashEdit extends ApiBase {
 			// Page exists: get the merged content with the proposed change
 			$baseRev = Revision::newFromPageId( $page->getId(), $params['baserevid'] );
 			if ( !$baseRev ) {
-				$this->dieUsage( "No revision ID {$params['baserevid']}", 'missingrev' );
+				$this->dieWithError( [ 'apierror-nosuchrevid', $params['baserevid'] ] );
 			}
 			$currentRev = $page->getRevision();
 			if ( !$currentRev ) {
-				$this->dieUsage( "No current revision of page ID {$page->getId()}", 'missingrev' );
+				$this->dieWithError( [ 'apierror-missingrev-pageid', $page->getId() ], 'missingrev' );
 			}
 			// Merge in the new version of the section to get the proposed version
 			$editContent = $page->replaceSectionAtRev(
@@ -105,7 +113,7 @@ class ApiStashEdit extends ApiBase {
 				$baseRev->getId()
 			);
 			if ( !$editContent ) {
-				$this->dieUsage( 'Could not merge updated section.', 'replacefailed' );
+				$this->dieWithError( 'apierror-sectionreplacefailed', 'replacefailed' );
 			}
 			if ( $currentRev->getId() == $baseRev->getId() ) {
 				// Base revision was still the latest; nothing to merge
@@ -115,7 +123,7 @@ class ApiStashEdit extends ApiBase {
 				$baseContent = $baseRev->getContent();
 				$currentContent = $currentRev->getContent();
 				if ( !$baseContent || !$currentContent ) {
-					$this->dieUsage( "Missing content for page ID {$page->getId()}", 'missingrev' );
+					$this->dieWithError( [ 'apierror-missingcontent-pageid', $page->getId() ], 'missingrev' );
 				}
 				$handler = ContentHandler::getForModelID( $baseContent->getModel() );
 				$content = $handler->merge3( $baseContent, $editContent, $currentContent );
@@ -160,7 +168,7 @@ class ApiStashEdit extends ApiBase {
 	 * @param Content $content Edit content
 	 * @param User $user
 	 * @param string $summary Edit summary
-	 * @return integer ApiStashEdit::ERROR_* constant
+	 * @return string ApiStashEdit::ERROR_* constant
 	 * @since 1.25
 	 */
 	public static function parseAndStash( WikiPage $page, Content $content, User $user, $summary ) {
