@@ -21,6 +21,8 @@
 
 namespace MediaWiki\Auth;
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * This represents the intention to set a temporary password for the user.
  * @ingroup Auth
@@ -33,12 +35,6 @@ class TemporaryPasswordAuthenticationRequest extends AuthenticationRequest {
 	/** @var bool Email password to the user. */
 	public $mailpassword = false;
 
-	/**
-	 * @var bool Do not fail certain operations if the password cannot be mailed, there is a
-	 *   backchannel present.
-	 */
-	public $hasBackchannel = false;
-
 	/** @var string Username or IP address of the caller */
 	public $caller;
 
@@ -67,7 +63,7 @@ class TemporaryPasswordAuthenticationRequest extends AuthenticationRequest {
 	 * @return TemporaryPasswordAuthenticationRequest
 	 */
 	public static function newRandom() {
-		$config = \ConfigFactory::getDefaultInstance()->makeConfig( 'main' );
+		$config = MediaWikiServices::getInstance()->getMainConfig();
 
 		// get the min password length
 		$minLength = $config->get( 'MinimalPasswordLength' );
