@@ -92,7 +92,7 @@ class TextContent extends AbstractContent {
 	 * Returns true if this content is not a redirect, and $wgArticleCountMethod
 	 * is "any".
 	 *
-	 * @param bool $hasLinks If it is known whether this content contains links,
+	 * @param bool|null $hasLinks If it is known whether this content contains links,
 	 * provide this information here, to avoid redundant parsing to find out.
 	 *
 	 * @return bool
@@ -147,9 +147,28 @@ class TextContent extends AbstractContent {
 		}
 	}
 
+	/**
+	 * Do a "\r\n" -> "\n" and "\r" -> "\n" transformation
+	 * as well as trim trailing whitespace
+	 *
+	 * This was formerly part of Parser::preSaveTransform, but
+	 * for non-wikitext content models they probably still want
+	 * to normalize line endings without all of the other PST
+	 * changes.
+	 *
+	 * @since 1.28
+	 * @param $text
+	 * @return string
+	 */
+	public static function normalizeLineEndings( $text ) {
+		return str_replace( [ "\r\n", "\r" ], "\n", rtrim( $text ) );
+	}
+
 	/**
 	 * Returns a Content object with pre-save transformations applied.
-	 * This implementation just trims trailing whitespace.
+	 *
+	 * At a minimum, subclasses should make sure to call TextContent::normalizeLineEndings()
+	 * either directly or part of Parser::preSaveTransform().
 	 *
 	 * @param Title $title
 	 * @param User $user
@@ -159,7 +178,7 @@ class TextContent extends AbstractContent {
 	 */
 	public function preSaveTransform( Title $title, User $user, ParserOptions $popts ) {
 		$text = $this->getNativeData();
-		$pst = rtrim( $text );
+		$pst = self::normalizeLineEndings( $text );
 
 		return ( $text === $pst ) ? $this : new static( $pst, $this->getModel() );
 	}
