@@ -20,12 +20,16 @@
 
 use MediaWiki\Logger\LoggerFactory;
 use MediaWiki\MediaWikiServices;
+use Psr\Log\LogLevel;
+use Wikimedia\Rdbms\DBError;
 
 /**
  * Handler class for MWExceptions
  * @ingroup Exception
  */
 class MWExceptionHandler {
+	const CAUGHT_BY_HANDLER = 'mwe_handler'; // error reported by this exception handler
+	const CAUGHT_BY_OTHER = 'other'; // error reported by direct logException() call
 
 	/**
 	 * @var string $reservedMemory
@@ -79,29 +83,32 @@ class MWExceptionHandler {
 	}
 
 	/**
-	 * If there are any open database transactions, roll them back and log
-	 * the stack trace of the exception that should have been caught so the
-	 * transaction could be aborted properly.
+	 * Roll back any open database transactions and log the stack trace of the exception
+	 *
+	 * This method is used to attempt to recover from exceptions
 	 *
 	 * @since 1.23
 	 * @param Exception|Throwable $e
 	 */
 	public static function rollbackMasterChangesAndLog( $e ) {
 		$services = MediaWikiServices::getInstance();
-		if ( $services->isServiceDisabled( 'DBLoadBalancerFactory' ) ) {
-			return; // T147599
+		if ( !$services->isServiceDisabled( 'DBLoadBalancerFactory' ) ) {
+			// Rollback DBs to avoid transaction notices. This might fail
+			// to rollback some databases due to connection issues or exceptions.
+			// However, any sane DB driver will rollback implicitly anyway.
+			try {
+				$services->getDBLoadBalancerFactory()->rollbackMasterChanges( __METHOD__ );
+			} catch ( DBError $e2 ) {
+				// If the DB is unreacheable, rollback() will throw an error
+				// and the error report() method might need messages from the DB,
+				// which would result in an exception loop. PHP may escalate such
+				// errors to "Exception thrown without a stack frame" fatals, but
+				// it's better to be explicit here.
+				self::logException( $e2, self::CAUGHT_BY_HANDLER );
+			}
 		}
 
-		$lbFactory = $services->getDBLoadBalancerFactory();
-		if ( $lbFactory->hasMasterChanges() ) {
-			$logger = LoggerFactory::getInstance( 'Bug56269' );
-			$logger->warning(
-				'Exception thrown with an uncommited database transaction: ' .
-				self::getLogMessage( $e ),
-				self::getLogContext( $e )
-			);
-		}
-		$lbFactory->rollbackMasterChanges( __METHOD__ );
+		self::logException( $e, self::CAUGHT_BY_HANDLER );
 	}
 
 	/**
@@ -119,23 +126,8 @@ class MWExceptionHandler {
 	 * @param Exception|Throwable $e
 	 */
 	public static function handleException( $e ) {
-		try {
-			// Rollback DBs to avoid transaction notices. This may fail
-			// to rollback some DB due to connection issues or exceptions.
-			// However, any sane DB driver will rollback implicitly anyway.
-			self::rollbackMasterChangesAndLog( $e );
-		} catch ( DBError $e2 ) {
-			// If the DB is unreacheable, rollback() will throw an error
-			// and the error report() method might need messages from the DB,
-			// which would result in an exception loop. PHP may escalate such
-			// errors to "Exception thrown without a stack frame" fatals, but
-			// it's better to be explicit here.
-			self::logException( $e2 );
-		}
-
-		self::logException( $e );
+		self::rollbackMasterChangesAndLog( $e );
 		self::report( $e );
-
 		// Exit value should be nonzero for the benefit of shell jobs
 		exit( 1 );
 	}
@@ -172,31 +164,37 @@ class MWExceptionHandler {
 		switch ( $level ) {
 			case E_RECOVERABLE_ERROR:
 				$levelName = 'Error';
+				$severity = LogLevel::ERROR;
 				break;
 			case E_WARNING:
 			case E_CORE_WARNING:
 			case E_COMPILE_WARNING:
 			case E_USER_WARNING:
 				$levelName = 'Warning';
+				$severity = LogLevel::WARNING;
 				break;
 			case E_NOTICE:
 			case E_USER_NOTICE:
 				$levelName = 'Notice';
+				$severity = LogLevel::INFO;
 				break;
 			case E_STRICT:
 				$levelName = 'Strict Standards';
+				$severity = LogLevel::DEBUG;
 				break;
 			case E_DEPRECATED:
 			case E_USER_DEPRECATED:
 				$levelName = 'Deprecated';
+				$severity = LogLevel::INFO;
 				break;
 			default:
 				$levelName = 'Unknown error';
+				$severity = LogLevel::ERROR;
 				break;
 		}
 
 		$e = new ErrorException( "PHP $levelName: $message", 0, $level, $file, $line );
-		self::logError( $e, 'error' );
+		self::logError( $e, 'error', $severity );
 
 		// This handler is for logging only. Return false will instruct PHP
 		// to continue regular handling.
@@ -284,7 +282,7 @@ TXT;
 		$trace = $trace ?: debug_backtrace();
 		$logger = LoggerFactory::getInstance( 'fatal' );
 		$logger->error( $msg, [
-			'exception' => [
+			'fatal_exception' => [
 				'class' => 'ErrorException',
 				'message' => "PHP Fatal Error: {$message}",
 				'code' => $level,
@@ -293,6 +291,7 @@ TXT;
 				'trace' => static::redactTrace( $trace ),
 			],
 			'exception_id' => wfRandomString( 8 ),
+			'caught_by' => self::CAUGHT_BY_HANDLER
 		] );
 
 		// Remember call so we don't double process via HHVM's fatal
@@ -332,7 +331,7 @@ TXT;
 				$text .= "{$pad}#{$level} {$frame['file']}({$frame['line']}): ";
 			} else {
 				// 'file' and 'line' are unset for calls via call_user_func
-				// (bug 55634) This matches behaviour of
+				// (T57634) This matches behaviour of
 				// Exception::getTraceAsString to instead display "[internal
 				// function]".
 				$text .= "{$pad}#{$level} [internal function]: ";
@@ -464,12 +463,14 @@ TXT;
 	 * logger.
 	 *
 	 * @param Exception|Throwable $e
+	 * @param string $catcher CAUGHT_BY_* class constant indicating what caught the error
 	 * @return array
 	 */
-	public static function getLogContext( $e ) {
+	public static function getLogContext( $e, $catcher = self::CAUGHT_BY_OTHER ) {
 		return [
 			'exception' => $e,
 			'exception_id' => WebRequest::getRequestId(),
+			'caught_by' => $catcher
 		];
 	}
 
@@ -481,11 +482,13 @@ TXT;
 	 * will be redacted as per getRedactedTraceAsArray().
 	 *
 	 * @param Exception|Throwable $e
+	 * @param string $catcher CAUGHT_BY_* class constant indicating what caught the error
 	 * @return array
 	 * @since 1.26
 	 */
-	public static function getStructuredExceptionData( $e ) {
+	public static function getStructuredExceptionData( $e, $catcher = self::CAUGHT_BY_OTHER ) {
 		global $wgLogExceptionBacktrace;
+
 		$data = [
 			'id' => WebRequest::getRequestId(),
 			'type' => get_class( $e ),
@@ -494,6 +497,7 @@ TXT;
 			'message' => $e->getMessage(),
 			'code' => $e->getCode(),
 			'url' => self::getURL() ?: null,
+			'caught_by' => $catcher
 		];
 
 		if ( $e instanceof ErrorException &&
@@ -509,7 +513,7 @@ TXT;
 
 		$previous = $e->getPrevious();
 		if ( $previous !== null ) {
-			$data['previous'] = self::getStructuredExceptionData( $previous );
+			$data['previous'] = self::getStructuredExceptionData( $previous, $catcher );
 		}
 
 		return $data;
@@ -566,11 +570,17 @@ TXT;
 	 * @param Exception|Throwable $e
 	 * @param bool $pretty Add non-significant whitespace to improve readability (default: false).
 	 * @param int $escaping Bitfield consisting of FormatJson::.*_OK class constants.
+	 * @param string $catcher CAUGHT_BY_* class constant indicating what caught the error
 	 * @return string|false JSON string if successful; false upon failure
 	 */
-	public static function jsonSerializeException( $e, $pretty = false, $escaping = 0 ) {
-		$data = self::getStructuredExceptionData( $e );
-		return FormatJson::encode( $data, $pretty, $escaping );
+	public static function jsonSerializeException(
+		$e, $pretty = false, $escaping = 0, $catcher = self::CAUGHT_BY_OTHER
+	) {
+		return FormatJson::encode(
+			self::getStructuredExceptionData( $e, $catcher ),
+			$pretty,
+			$escaping
+		);
 	}
 
 	/**
@@ -581,16 +591,17 @@ TXT;
 	 *
 	 * @since 1.22
 	 * @param Exception|Throwable $e
+	 * @param string $catcher CAUGHT_BY_* class constant indicating what caught the error
 	 */
-	public static function logException( $e ) {
+	public static function logException( $e, $catcher = self::CAUGHT_BY_OTHER ) {
 		if ( !( $e instanceof MWException ) || $e->isLoggable() ) {
 			$logger = LoggerFactory::getInstance( 'exception' );
 			$logger->error(
 				self::getLogMessage( $e ),
-				self::getLogContext( $e )
+				self::getLogContext( $e, $catcher )
 			);
 
-			$json = self::jsonSerializeException( $e, false, FormatJson::ALL_OK );
+			$json = self::jsonSerializeException( $e, false, FormatJson::ALL_OK, $catcher );
 			if ( $json !== false ) {
 				$logger = LoggerFactory::getInstance( 'exception-json' );
 				$logger->error( $json, [ 'private' => true ] );
@@ -606,25 +617,30 @@ TXT;
 	 * @since 1.25
 	 * @param ErrorException $e
 	 * @param string $channel
+	 * @param string $level
 	*/
-	protected static function logError( ErrorException $e, $channel ) {
+	protected static function logError(
+		ErrorException $e, $channel, $level = LogLevel::ERROR
+	) {
+		$catcher = self::CAUGHT_BY_HANDLER;
 		// The set_error_handler callback is independent from error_reporting.
 		// Filter out unwanted errors manually (e.g. when
 		// MediaWiki\suppressWarnings is active).
 		$suppressed = ( error_reporting() & $e->getSeverity() ) === 0;
 		if ( !$suppressed ) {
 			$logger = LoggerFactory::getInstance( $channel );
-			$logger->error(
+			$logger->log(
+				$level,
 				self::getLogMessage( $e ),
-				self::getLogContext( $e )
+				self::getLogContext( $e, $catcher )
 			);
 		}
 
 		// Include all errors in the json log (surpressed errors will be flagged)
-		$json = self::jsonSerializeException( $e, false, FormatJson::ALL_OK );
+		$json = self::jsonSerializeException( $e, false, FormatJson::ALL_OK, $catcher );
 		if ( $json !== false ) {
 			$logger = LoggerFactory::getInstance( "{$channel}-json" );
-			$logger->error( $json, [ 'private' => true ] );
+			$logger->log( $level, $json, [ 'private' => true ] );
 		}
 
 		Hooks::run( 'LogException', [ $e, $suppressed ] );
