@@ -22,6 +22,10 @@
  * @author Aaron Schulz
  */
 
+use MediaWiki\MediaWikiServices;
+use Wikimedia\Rdbms\IDatabase;
+use Wikimedia\Rdbms\DBError;
+
 /**
  * Version of FileJournal that logs to a DB table
  * @since 1.20
@@ -180,7 +184,7 @@ class DBFileJournal extends FileJournal {
 	protected function getMasterDB() {
 		if ( !$this->dbw ) {
 			// Get a separate connection in autocommit mode
-			$lb = wfGetLBFactory()->newMainLB();
+			$lb =  MediaWikiServices::getInstance()->getDBLoadBalancerFactory()->newMainLB();
 			$this->dbw = $lb->getConnection( DB_MASTER, [], $this->wiki );
 			$this->dbw->clearFlag( DBO_TRX );
 		}
