@@ -21,6 +21,8 @@
  * @ingroup FileAbstraction
  */
 
+use Wikimedia\Rdbms\DBUnexpectedError;
+
 /**
  * Foreign file with an accessible MediaWiki database
  *
@@ -120,10 +122,10 @@ class ForeignDBFile extends LocalFile {
 	}
 
 	/**
-	 * @param bool|Language $lang Optional language to fetch description in.
-	 * @return string
+	 * @param Language|null $lang Optional language to fetch description in.
+	 * @return string|false
 	 */
-	function getDescriptionText( $lang = false ) {
+	function getDescriptionText( $lang = null ) {
 		global $wgLang;
 
 		if ( !$this->repo->fetchDescription ) {
@@ -136,7 +138,7 @@ class ForeignDBFile extends LocalFile {
 			return false;
 		}
 
-		$touched = $this->repo->getSlaveDB()->selectField(
+		$touched = $this->repo->getReplicaDB()->selectField(
 			'page',
 			'page_touched',
 			[
@@ -179,7 +181,7 @@ class ForeignDBFile extends LocalFile {
 	 * @since 1.27
 	 */
 	public function getDescriptionShortUrl() {
-		$dbr = $this->repo->getSlaveDB();
+		$dbr = $this->repo->getReplicaDB();
 		$pageId = $dbr->selectField(
 			'page',
 			'page_id',
