@@ -21,6 +21,10 @@
  * @ingroup Deployment
  */
 
+use Wikimedia\Rdbms\Database;
+use Wikimedia\Rdbms\DBQueryError;
+use Wikimedia\Rdbms\DBConnectionError;
+
 /**
  * Class for setting up the MediaWiki database using Microsoft SQL Server.
  *
@@ -214,6 +218,7 @@ class MssqlInstaller extends DatabaseInstaller {
 		try {
 			$db = Database::factory( 'mssql', [
 				'host' => $this->getVar( 'wgDBserver' ),
+				'port' => $this->getVar( 'wgDBport' ),
 				'user' => $user,
 				'password' => $password,
 				'dbname' => false,
@@ -282,7 +287,7 @@ class MssqlInstaller extends DatabaseInstaller {
 		// Check to ensure we can grant everything needed as well
 		// We can't actually tell if we have WITH GRANT OPTION for a given permission, so we assume we do
 		// and just check for the permission
-		// http://technet.microsoft.com/en-us/library/ms178569.aspx
+		// https://technet.microsoft.com/en-us/library/ms178569.aspx
 		// The following array sets up which permissions imply whatever permissions we specify
 		$implied = [
 			// schema           database  server
@@ -572,7 +577,7 @@ class MssqlInstaller extends DatabaseInstaller {
 					$grantableNames[] = $dbUser;
 				} catch ( DBQueryError $dqe ) {
 					$this->db->rollback();
-					$status->warning( 'config-install-user-create-failed', $dbUser, $dqe->getText() );
+					$status->warning( 'config-install-user-create-failed', $dbUser, $dqe->getMessage() );
 				}
 			} elseif ( !$this->userExists( $dbUser ) ) {
 				try {
@@ -583,7 +588,7 @@ class MssqlInstaller extends DatabaseInstaller {
 					$grantableNames[] = $dbUser;
 				} catch ( DBQueryError $dqe ) {
 					$this->db->rollback();
-					$status->warning( 'config-install-user-create-failed', $dbUser, $dqe->getText() );
+					$status->warning( 'config-install-user-create-failed', $dbUser, $dqe->getMessage() );
 				}
 			} else {
 				$status->warning( 'config-install-user-alreadyexists', $dbUser );
@@ -615,7 +620,7 @@ class MssqlInstaller extends DatabaseInstaller {
 					$this->db->commit();
 				} catch ( DBQueryError $dqe ) {
 					$this->db->rollback();
-					$status->fatal( 'config-install-user-grant-failed', $dbUser, $dqe->getText() );
+					$status->fatal( 'config-install-user-grant-failed', $dbUser, $dqe->getMessage() );
 				}
 				// Also try to grant SHOWPLAN on the db, but don't fail if we can't
 				// (just makes a couple things in mediawiki run slower since
@@ -641,7 +646,7 @@ class MssqlInstaller extends DatabaseInstaller {
 				$this->db->query( "CREATE FULLTEXT INDEX ON $searchindex (si_title, si_text) "
 					. "KEY INDEX si_page ON $schema" );
 			} catch ( DBQueryError $dqe ) {
-				$status->fatal( 'config-install-tables-failed', $dqe->getText() );
+				$status->fatal( 'config-install-tables-failed', $dqe->getMessage() );
 			}
 		}
 
