@@ -23,6 +23,8 @@
  * @since 1.22
  */
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * This class formats delete log entries.
  *
@@ -37,6 +39,12 @@ class DeleteLogFormatter extends LogFormatter {
 				// logentry-suppress-event-legacy, logentry-suppress-revision-legacy
 				return "$key-legacy";
 			}
+		} elseif ( $this->entry->getSubtype() === 'restore' ) {
+			$rawParams = $this->entry->getParameters();
+			if ( !isset( $rawParams[':assoc:count'] ) ) {
+				// Message: logentry-delete-restore-nocount
+				return $key . '-nocount';
+			}
 		}
 
 		return $key;
@@ -54,12 +62,14 @@ class DeleteLogFormatter extends LogFormatter {
 			// 'filearchive' for file versions, or a comma-separated list of log_ids for log
 			// entries. $subtype here is 'revision' for page revisions and file
 			// versions, or 'event' for log entries.
-			if ( ( $subtype === 'event' && count( $params ) === 6 )
-				|| ( $subtype === 'revision' && isset( $params[3] )
-					&& ( $params[3] === 'revision' || $params[3] === 'oldimage'
-						|| $params[3] === 'archive' || $params[3] === 'filearchive' )
+			if (
+				( $subtype === 'event' && count( $params ) === 6 )
+				|| (
+					$subtype === 'revision' && isset( $params[3] )
+					&& in_array( $params[3], [ 'revision', 'archive', 'oldimage', 'filearchive' ] )
 				)
 			) {
+				// See RevDelList::getLogParams()/RevDelLogList::getLogParams()
 				$paramStart = $subtype === 'revision' ? 4 : 3;
 
 				$old = $this->parseBitField( $params[$paramStart + 1] );
@@ -70,7 +80,8 @@ class DeleteLogFormatter extends LogFormatter {
 				foreach ( $hid as $v ) {
 					$changes[] = $this->msg( "$v-hid" )->plain();
 				}
-				// messages used: revdelete-content-unhid, revdelete-summary-unhid, revdelete-uname-unhid
+				// messages used: revdelete-content-unhid, revdelete-summary-unhid,
+				// revdelete-uname-unhid
 				foreach ( $unhid as $v ) {
 					$changes[] = $this->msg( "$v-unhid" )->plain();
 				}
@@ -92,6 +103,19 @@ class DeleteLogFormatter extends LogFormatter {
 				$this->parsedParametersDeleteLog = array_slice( $params, 0, 3 );
 				return $this->parsedParametersDeleteLog;
 			}
+		} elseif ( $subtype === 'restore' ) {
+			$rawParams = $this->entry->getParameters();
+			if ( isset( $rawParams[':assoc:count'] ) ) {
+				$countList = [];
+				foreach ( $rawParams[':assoc:count'] as $type => $count ) {
+					if ( $count ) {
+						// Messages: restore-count-revisions, restore-count-files
+						$countList[] = $this->context->msg( 'restore-count-' . $type )
+							->numParams( $count )->plain();
+					}
+				}
+				$params[3] = $this->context->getLanguage()->listToText( $countList );
+			}
 		}
 
 		$this->parsedParametersDeleteLog = $params;
@@ -111,6 +135,7 @@ class DeleteLogFormatter extends LogFormatter {
 
 	public function getActionLinks() {
 		$user = $this->context->getUser();
+		$linkRenderer = MediaWikiServices::getInstance()->getLinkRenderer();
 		if ( !$user->isAllowed( 'deletedhistory' )
 			|| $this->entry->isDeleted( LogPage::DELETED_ACTION )
 		) {
@@ -119,14 +144,15 @@ class DeleteLogFormatter extends LogFormatter {
 
 		switch ( $this->entry->getSubtype() ) {
 			case 'delete': // Show undelete link
+			case 'delete_redir':
 				if ( $user->isAllowed( 'undelete' ) ) {
 					$message = 'undeletelink';
 				} else {
 					$message = 'undeleteviewlink';
 				}
-				$revert = Linker::linkKnown(
+				$revert = $linkRenderer->makeKnownLink(
 					SpecialPage::getTitleFor( 'Undelete' ),
-					$this->msg( $message )->escaped(),
+					$this->msg( $message )->text(),
 					[],
 					[ 'target' => $this->entry->getTarget()->getPrefixedDBkey() ]
 				);
@@ -152,9 +178,9 @@ class DeleteLogFormatter extends LogFormatter {
 				if ( count( $ids ) == 1 ) {
 					// Live revision diffs...
 					if ( $key == 'oldid' || $key == 'revision' ) {
-						$links[] = Linker::linkKnown(
+						$links[] = $linkRenderer->makeKnownLink(
 							$this->entry->getTarget(),
-							$this->msg( 'diff' )->escaped(),
+							$this->msg( 'diff' )->text(),
 							[],
 							[
 								'diff' => intval( $ids[0] ),
@@ -163,9 +189,9 @@ class DeleteLogFormatter extends LogFormatter {
 						);
 						// Deleted revision diffs...
 					} elseif ( $key == 'artimestamp' || $key == 'archive' ) {
-						$links[] = Linker::linkKnown(
+						$links[] = $linkRenderer->makeKnownLink(
 							SpecialPage::getTitleFor( 'Undelete' ),
-							$this->msg( 'diff' )->escaped(),
+							$this->msg( 'diff' )->text(),
 							[],
 							[
 								'target' => $this->entry->getTarget()->getPrefixedDBkey(),
@@ -177,9 +203,9 @@ class DeleteLogFormatter extends LogFormatter {
 				}
 
 				// View/modify link...
-				$links[] = Linker::linkKnown(
+				$links[] = $linkRenderer->makeKnownLink(
 					SpecialPage::getTitleFor( 'Revisiondelete' ),
-					$this->msg( 'revdel-restore' )->escaped(),
+					$this->msg( 'revdel-restore' )->text(),
 					[],
 					[
 						'target' => $this->entry->getTarget()->getPrefixedText(),
@@ -202,9 +228,9 @@ class DeleteLogFormatter extends LogFormatter {
 					$query = implode( ',', $query );
 				}
 				// Link to each hidden object ID, $params[1] is the url param
-				$revert = Linker::linkKnown(
+				$revert = $linkRenderer->makeKnownLink(
 					SpecialPage::getTitleFor( 'Revisiondelete' ),
-					$this->msg( 'revdel-restore' )->escaped(),
+					$this->msg( 'revdel-restore' )->text(),
 					[],
 					[
 						'target' => $this->entry->getTarget()->getPrefixedText(),
@@ -269,6 +295,11 @@ class DeleteLogFormatter extends LogFormatter {
 				$params[':assoc:old'][$key] = (bool)( $old & $bit );
 				$params[':assoc:new'][$key] = (bool)( $new & $bit );
 			}
+		} elseif ( $subtype === 'restore' ) {
+			$rawParams = $entry->getParameters();
+			if ( isset( $rawParams[':assoc:count'] ) ) {
+				$params[':assoc:count'] = $rawParams[':assoc:count'];
+			}
 		}
 
 		return $params;
