@@ -19,6 +19,8 @@
  * @ingroup RevisionDelete
  */
 
+use Wikimedia\Rdbms\IDatabase;
+
 /**
  * Item class for an oldimage table row
  */
@@ -116,19 +118,19 @@ class RevDelFileItem extends RevDelItem {
 	 * @return string
 	 */
 	protected function getLink() {
-		$date = htmlspecialchars( $this->list->getLanguage()->userTimeAndDate(
-			$this->file->getTimestamp(), $this->list->getUser() ) );
+		$date = $this->list->getLanguage()->userTimeAndDate(
+			$this->file->getTimestamp(), $this->list->getUser() );
 
 		if ( !$this->isDeleted() ) {
 			# Regular files...
-			return Html::rawElement( 'a', [ 'href' => $this->file->getUrl() ], $date );
+			return Html::element( 'a', [ 'href' => $this->file->getUrl() ], $date );
 		}
 
 		# Hidden files...
 		if ( !$this->canViewContent() ) {
-			$link = $date;
+			$link = htmlspecialchars( $date );
 		} else {
-			$link = Linker::link(
+			$link = $this->getLinkRenderer()->makeLink(
 				SpecialPage::getTitleFor( 'Revisiondelete' ),
 				$date,
 				[],
@@ -202,10 +204,10 @@ class RevDelFileItem extends RevDelItem {
 			'width' => $file->getWidth(),
 			'height' => $file->getHeight(),
 			'size' => $file->getSize(),
+			'userhidden' => (bool)$file->isDeleted( Revision::DELETED_USER ),
+			'commenthidden' => (bool)$file->isDeleted( Revision::DELETED_COMMENT ),
+			'contenthidden' => (bool)$this->isDeleted(),
 		];
-		$ret += $file->isDeleted( Revision::DELETED_USER ) ? [ 'userhidden' => '' ] : [];
-		$ret += $file->isDeleted( Revision::DELETED_COMMENT ) ? [ 'commenthidden' => '' ] : [];
-		$ret += $this->isDeleted() ? [ 'contenthidden' => '' ] : [];
 		if ( !$this->isDeleted() ) {
 			$ret += [
 				'url' => $file->getUrl(),
