@@ -91,21 +91,21 @@ class Site implements Serializable {
 	 *
 	 * @var array[]
 	 */
-	protected $localIds = array();
+	protected $localIds = [];
 
 	/**
 	 * @since 1.21
 	 *
 	 * @var array
 	 */
-	protected $extraData = array();
+	protected $extraData = [];
 
 	/**
 	 * @since 1.21
 	 *
 	 * @var array
 	 */
-	protected $extraConfig = array();
+	protected $extraConfig = [];
 
 	/**
 	 * @since 1.21
@@ -172,7 +172,7 @@ class Site implements Serializable {
 	}
 
 	/**
-	 * Gets the type of the site (ie wikipedia).
+	 * Gets the group of the site (ie wikipedia).
 	 *
 	 * @since 1.21
 	 *
@@ -183,7 +183,7 @@ class Site implements Serializable {
 	}
 
 	/**
-	 * Sets the type of the site (ie wikipedia).
+	 * Sets the group of the site (ie wikipedia).
 	 *
 	 * @since 1.21
 	 *
@@ -463,6 +463,9 @@ class Site implements Serializable {
 	 * @param string $languageCode
 	 */
 	public function setLanguageCode( $languageCode ) {
+		if ( !Language::isValidCode( $languageCode ) ) {
+			throw new InvalidArgumentException( "$languageCode is not a valid language code." );
+		}
 		$this->languageCode = $languageCode;
 	}
 
@@ -499,11 +502,11 @@ class Site implements Serializable {
 	 */
 	public function addLocalId( $type, $identifier ) {
 		if ( $this->localIds === false ) {
-			$this->localIds = array();
+			$this->localIds = [];
 		}
 
 		if ( !array_key_exists( $type, $this->localIds ) ) {
-			$this->localIds[$type] = array();
+			$this->localIds[$type] = [];
 		}
 
 		if ( !in_array( $identifier, $this->localIds[$type] ) ) {
@@ -543,7 +546,7 @@ class Site implements Serializable {
 	public function getInterwikiIds() {
 		return array_key_exists( self::ID_INTERWIKI, $this->localIds )
 			? $this->localIds[self::ID_INTERWIKI]
-			: array();
+			: [];
 	}
 
 	/**
@@ -557,7 +560,7 @@ class Site implements Serializable {
 	public function getNavigationIds() {
 		return array_key_exists( self::ID_EQUIVALENT, $this->localIds )
 			? $this->localIds[self::ID_EQUIVALENT] :
-			array();
+			[];
 	}
 
 	/**
@@ -588,7 +591,7 @@ class Site implements Serializable {
 		}
 
 		if ( !array_key_exists( 'paths', $this->extraData ) ) {
-			$this->extraData['paths'] = array();
+			$this->extraData['paths'] = [];
 		}
 
 		$this->extraData['paths'][$pathType] = $fullUrl;
@@ -617,7 +620,7 @@ class Site implements Serializable {
 	 * @return string[]
 	 */
 	public function getAllPaths() {
-		return array_key_exists( 'paths', $this->extraData ) ? $this->extraData['paths'] : array();
+		return array_key_exists( 'paths', $this->extraData ) ? $this->extraData['paths'] : [];
 	}
 
 	/**
@@ -658,7 +661,7 @@ class Site implements Serializable {
 	 * @return string
 	 */
 	public function serialize() {
-		$fields = array(
+		$fields = [
 			'globalid' => $this->globalId,
 			'type' => $this->type,
 			'group' => $this->group,
@@ -670,7 +673,7 @@ class Site implements Serializable {
 			'forward' => $this->forward,
 			'internalid' => $this->internalId,
 
-		);
+		];
 
 		return serialize( $fields );
 	}
@@ -698,9 +701,3 @@ class Site implements Serializable {
 		$this->setInternalId( $fields['internalid'] );
 	}
 }
-
-/**
- * @deprecated since 1.21
- */
-class SiteObject extends Site {
-}
