@@ -149,6 +149,9 @@ abstract class Skin extends ContextSource {
 	 * Defines the ResourceLoader modules that should be added to the skin
 	 * It is recommended that skins wishing to override call parent::getDefaultModules()
 	 * and substitute out any modules they wish to change by using a key to look them up
+	 *
+	 * For style modules, use setupSkinUserCss() instead.
+	 *
 	 * @return array Array of modules with helper keys for easy overriding
 	 */
 	public function getDefaultModules() {
@@ -161,8 +164,6 @@ abstract class Skin extends ContextSource {
 			'content' => [
 				'mediawiki.page.ready',
 			],
-			// modules that exist for legacy reasons
-			'legacy' => ResourceLoaderStartUpModule::getLegacyModules(),
 			// modules relating to search functionality
 			'search' => [],
 			// modules relating to functionality relating to watching an article
@@ -171,6 +172,16 @@ abstract class Skin extends ContextSource {
 			'user' => [],
 		];
 
+		// Preload jquery.tablesorter for mediawiki.page.ready
+		if ( strpos( $out->getHTML(), 'sortable' ) !== false ) {
+			$modules['content'][] = 'jquery.tablesorter';
+		}
+
+		// Preload jquery.makeCollapsible for mediawiki.page.ready
+		if ( strpos( $out->getHTML(), 'mw-collapsible' ) !== false ) {
+			$modules['content'][] = 'jquery.makeCollapsible';
+		}
+
 		// Add various resources if required
 		if ( $wgUseAjax && $wgEnableAPI ) {
 			if ( $wgEnableWriteAPI && $user->isLoggedIn()
@@ -217,6 +228,17 @@ abstract class Skin extends ContextSource {
 			}
 		}
 
+		// Footer links (used by SkinTemplate::prepareQuickTemplate)
+		foreach ( [
+			$this->footerLinkTitle( 'privacy', 'privacypage' ),
+			$this->footerLinkTitle( 'aboutsite', 'aboutpage' ),
+			$this->footerLinkTitle( 'disclaimers', 'disclaimerpage' ),
+		] as $title ) {
+			if ( $title ) {
+				$titles[] = $title;
+			}
+		}
+
 		Hooks::run( 'SkinPreloadExistence', [ &$titles, $this ] );
 
 		if ( $titles ) {
@@ -370,7 +392,7 @@ abstract class Skin extends ContextSource {
 
 		if ( $title->isSpecialPage() ) {
 			$type = 'ns-special';
-			// bug 23315: provide a class based on the canonical special page name without subpages
+			// T25315: provide a class based on the canonical special page name without subpages
 			list( $canonicalName ) = SpecialPageFactory::resolveAlias( $title->getDBkey() );
 			if ( $canonicalName ) {
 				$type .= ' ' . Sanitizer::escapeClass( "mw-special-$canonicalName" );
@@ -895,7 +917,10 @@ abstract class Skin extends ContextSource {
 				$html = htmlspecialchars( $icon["alt"] );
 			}
 			if ( $url ) {
-				$html = Html::rawElement( 'a', [ "href" => $url ], $html );
+				global $wgExternalLinkTarget;
+				$html = Html::rawElement( 'a',
+					[ "href" => $url, "target" => $wgExternalLinkTarget ],
+					$html );
 			}
 		}
 		return $html;
@@ -921,25 +946,34 @@ abstract class Skin extends ContextSource {
 	 * @return string HTML anchor
 	 */
 	public function footerLink( $desc, $page ) {
-		// if the link description has been set to "-" in the default language,
+		$title = $this->footerLinkTitle( $desc, $page );
+		if ( !$title ) {
+			return '';
+		}
+
+		return Linker::linkKnown(
+			$title,
+			$this->msg( $desc )->escaped()
+		);
+	}
+
+	/**
+	 * @param string $desc
+	 * @param string $page
+	 * @return Title|null
+	 */
+	private function footerLinkTitle( $desc, $page ) {
+		// If the link description has been set to "-" in the default language,
 		if ( $this->msg( $desc )->inContentLanguage()->isDisabled() ) {
 			// then it is disabled, for all languages.
-			return '';
-		} else {
-			// Otherwise, we display the link for the user, described in their
-			// language (which may or may not be the same as the default language),
-			// but we make the link target be the one site-wide page.
-			$title = Title::newFromText( $this->msg( $page )->inContentLanguage()->text() );
-
-			if ( !$title ) {
-				return '';
-			}
-
-			return Linker::linkKnown(
-				$title,
-				$this->msg( $desc )->escaped()
-			);
+			return null;
 		}
+		// Otherwise, we display the link for the user, described in their
+		// language (which may or may not be the same as the default language),
+		// but we make the link target be the one site-wide page.
+		$title = Title::newFromText( $this->msg( $page )->inContentLanguage()->text() );
+
+		return $title ?: null;
 	}
 
 	/**
@@ -1015,7 +1049,7 @@ abstract class Skin extends ContextSource {
 		global $wgStylePath, $wgStyleVersion;
 
 		if ( $this->stylename === null ) {
-			$class = get_class( $this );
+			$class = static::class;
 			throw new MWException( "$class::\$stylename must be set to use getSkinStylePath()" );
 		}
 
@@ -1263,7 +1297,7 @@ abstract class Skin extends ContextSource {
 					$line = array_map( 'trim', explode( '|', $line, 2 ) );
 					if ( count( $line ) !== 2 ) {
 						// Second sanity check, could be hit by people doing
-						// funky stuff with parserfuncs... (bug 33321)
+						// funky stuff with parserfuncs... (T35321)
 						continue;
 					}
 
@@ -1518,7 +1552,7 @@ abstract class Skin extends ContextSource {
 
 		$attribs = [];
 		if ( !is_null( $tooltip ) ) {
-			# Bug 25462: undo double-escaping.
+			# T27462: undo double-escaping.
 			$tooltip = Sanitizer::decodeCharReferences( $tooltip );
 			$attribs['title'] = wfMessage( 'editsectionhint' )->rawParams( $tooltip )
 				->inLanguage( $lang )->text();
