@@ -69,7 +69,11 @@ class SpecialAllPages extends IncludableSpecialPage {
 		$from = $request->getVal( 'from', null );
 		$to = $request->getVal( 'to', null );
 		$namespace = $request->getInt( 'namespace' );
-		$hideredirects = $request->getBool( 'hideredirects', false );
+
+		$miserMode = (bool)$this->getConfig()->get( 'MiserMode' );
+
+		// Redirects filter is disabled in MiserMode
+		$hideredirects = $request->getBool( 'hideredirects', false ) && !$miserMode;
 
 		$namespaces = $this->getLanguage()->getNamespaces();
 
@@ -100,6 +104,7 @@ class SpecialAllPages extends IncludableSpecialPage {
 	protected function outputHTMLForm( $namespace = NS_MAIN,
 		$from = '', $to = '', $hideRedirects = false
 	) {
+		$miserMode = (bool)$this->getConfig()->get( 'MiserMode' );
 		$fields = [
 			'from' => [
 				'type' => 'text',
@@ -133,6 +138,11 @@ class SpecialAllPages extends IncludableSpecialPage {
 				'value' => $hideRedirects,
 			],
 		];
+
+		if ( $miserMode ) {
+			unset( $fields['hideredirects'] );
+		}
+
 		$form = HTMLForm::factory( 'table', $fields, $this->getContext() );
 		$form->setMethod( 'get' )
 			->setWrapperLegendMsg( 'allpages' )
@@ -204,6 +214,7 @@ class SpecialAllPages extends IncludableSpecialPage {
 				]
 			);
 
+			$linkRenderer = $this->getLinkRenderer();
 			if ( $res->numRows() > 0 ) {
 				$out = Html::openElement( 'ul', [ 'class' => 'mw-allpages-chunk' ] );
 
@@ -213,7 +224,7 @@ class SpecialAllPages extends IncludableSpecialPage {
 						$out .= '<li' .
 							( $s->page_is_redirect ? ' class="allpagesredirect"' : '' ) .
 							'>' .
-							Linker::link( $t ) .
+							$linkRenderer->makeLink( $t ) .
 							"</li>\n";
 					} else {
 						$out .= '<li>[[' . htmlspecialchars( $s->page_title ) . "]]</li>\n";
@@ -269,6 +280,7 @@ class SpecialAllPages extends IncludableSpecialPage {
 		$navLinks = [];
 		$self = $this->getPageTitle();
 
+		$linkRenderer = $this->getLinkRenderer();
 		// Generate a "previous page" link if needed
 		if ( $prevTitle ) {
 			$query = [ 'from' => $prevTitle->getText() ];
@@ -281,9 +293,9 @@ class SpecialAllPages extends IncludableSpecialPage {
 				$query['hideredirects'] = $hideredirects;
 			}
 
-			$navLinks[] = Linker::linkKnown(
+			$navLinks[] = $linkRenderer->makeKnownLink(
 				$self,
-				$this->msg( 'prevpage', $prevTitle->getText() )->escaped(),
+				$this->msg( 'prevpage', $prevTitle->getText() )->text(),
 				[],
 				$query
 			);
@@ -304,9 +316,9 @@ class SpecialAllPages extends IncludableSpecialPage {
 				$query['hideredirects'] = $hideredirects;
 			}
 
-			$navLinks[] = Linker::linkKnown(
+			$navLinks[] = $linkRenderer->makeKnownLink(
 				$self,
-				$this->msg( 'nextpage', $t->getText() )->escaped(),
+				$this->msg( 'nextpage', $t->getText() )->text(),
 				[],
 				$query
 			);
