@@ -31,7 +31,6 @@ class SearchInputWidget extends TitleInputWidget {
 	public function __construct( array $config = [] ) {
 		$config = array_merge( [
 			'maxLength' => null,
-			'type' => 'search',
 			'icon' => 'search',
 		], $config );
 
@@ -56,6 +55,10 @@ class SearchInputWidget extends TitleInputWidget {
 		$this->addClasses( [ 'mw-widget-searchInputWidget' ] );
 	}
 
+	protected function getInputElement( $config ) {
+		return ( new \OOUI\Tag( 'input' ) )->setAttributes( [ 'type' => 'search' ] );
+	}
+
 	protected function getJavaScriptClassName() {
 		return 'mw.widgets.SearchInputWidget';
 	}
