@@ -60,7 +60,7 @@ class Names {
 		'als' => 'Alemannisch', # Alemannic -- not a valid code, for compatibility. See gsw.
 		'am' => 'አማርኛ', # Amharic
 		'an' => 'aragonés', # Aragonese
-		'ang' => 'Ænglisc', # Old English, bug 23283
+		'ang' => 'Ænglisc', # Old English, T25283
 		'anp' => 'अङ्गिका', # Angika
 		'ar' => 'العربية', # Arabic
 		'arc' => 'ܐܪܡܝܐ', # Aramaic
@@ -71,6 +71,7 @@ class Names {
 		'as' => 'অসমীয়া', # Assamese
 		'ase' => 'American sign language', # American sign language
 		'ast' => 'asturianu', # Asturian
+		'atj' => 'Atikamekw', # Atikamekw
 		'av' => 'авар', # Avar
 		'avk' => 'Kotava', # Kotava
 		'awa' => 'अवधी', # Awadhi
@@ -131,6 +132,7 @@ class Names {
 		'de-at' => 'Österreichisches Deutsch', # Austrian German
 		'de-ch' => 'Schweizer Hochdeutsch', # Swiss Standard German
 		'de-formal' => "Deutsch (Sie-Form)\xE2\x80\x8E", # German - formal address ("Sie")
+		'din' => 'Thuɔŋjäŋ', # Dinka
 		'diq' => 'Zazaki', # Zazaki
 		'dsb' => 'dolnoserbski', # Lower Sorbian
 		'dtp' => 'Dusun Bundu-liwan', # Central Dusun
@@ -174,6 +176,7 @@ class Names {
 		'gom' => 'गोंयची कोंकणी / Gõychi Konknni', # Goan Konkani
 		'gom-deva' => 'गोंयची कोंकणी', # Goan Konkani (Devanagari script)
 		'gom-latn' => 'Gõychi Konknni', # Goan Konkani (Latin script)
+		'gor' => 'Bahasa Hulontalo', # Gorontalo
 		'got' => '𐌲𐌿𐍄𐌹𐍃𐌺', # Gothic
 		'grc' => 'Ἀρχαία ἑλληνικὴ', # Ancient Greek
 		'gsw' => 'Alemannisch', # Alemannic
@@ -241,6 +244,7 @@ class Names {
 		'krc' => 'къарачай-малкъар', # Karachay-Balkar
 		'kri' => 'Krio', # Krio
 		'krj' => 'Kinaray-a', # Kinaray-a
+		'krl' => 'karjal', # Karelian
 		'ks' => 'कॉशुर / کٲشُر', # Kashmiri (multiple scripts - defaults to Perso-Arabic)
 		'ks-arab' => 'کٲشُر', # Kashmiri (Perso-Arabic script)
 		'ks-deva' => 'कॉशुर', # Kashmiri (Devanagari script)
@@ -272,7 +276,7 @@ class Names {
 		'lus' => 'Mizo ţawng', # Mizo/Lushai
 		'luz' => 'لئری دوٙمینی', # Southern Luri
 		'lv' => 'latviešu', # Latvian
-		'lzh' => '文言', # Literary Chinese, bug 8217
+		'lzh' => '文言', # Literary Chinese, T10217
 		'lzz' => 'Lazuri', # Laz
 		'mai' => 'मैथिली', # Maithili
 		'map-bms' => 'Basa Banyumasan', # Banyumasan
@@ -297,8 +301,8 @@ class Names {
 		'mzn' => 'مازِرونی', # Mazanderani
 		'na' => 'Dorerin Naoero', # Nauruan
 		'nah' => 'Nāhuatl', # Nahuatl (not in ISO 639-3)
-		'nan' => 'Bân-lâm-gú', # Min-nan, bug 8217
-		'nap' => 'Napulitano', # Neapolitan, bug 43793
+		'nan' => 'Bân-lâm-gú', # Min-nan, T10217
+		'nap' => 'Napulitano', # Neapolitan, T45793
 		'nb' => 'norsk bokmål', # Norwegian (Bokmal)
 		'nds' => 'Plattdüütsch', # Low German ''or'' Low Saxon
 		'nds-nl' => 'Nedersaksies', # aka Nedersaksisch: Dutch Low Saxon
@@ -319,7 +323,7 @@ class Names {
 		'olo' => 'Livvinkarjala', # Livvi-Karelian
 		'om' => 'Oromoo', # Oromo
 		'or' => 'ଓଡ଼ିଆ', # Oriya
-		'os' => 'Ирон', # Ossetic, bug 29091
+		'os' => 'Ирон', # Ossetic, T31091
 		'pa' => 'ਪੰਜਾਬੀ', # Eastern Punjabi (Gurmukhi script) (pan)
 		'pag' => 'Pangasinan', # Pangasinan
 		'pam' => 'Kapampangan', # Pampanga
@@ -454,17 +458,17 @@ class Names {
 		'za' => 'Vahcuengh', # Zhuang
 		'zea' => 'Zeêuws', # Zeeuws/Zeaws
 		'zh' => '中文', # (Zhōng Wén) - Chinese
-		'zh-classical' => '文言', # Classical Chinese/Literary Chinese -- (see bug 8217)
+		'zh-classical' => '文言', # Classical Chinese/Literary Chinese -- (see T10217)
 		'zh-cn' => "中文（中国大陆）\xE2\x80\x8E", # Chinese (PRC)
 		'zh-hans' => "中文（简体）\xE2\x80\x8E", # Mandarin Chinese (Simplified Chinese script) (cmn-hans)
 		'zh-hant' => "中文（繁體）\xE2\x80\x8E", # Mandarin Chinese (Traditional Chinese script) (cmn-hant)
 		'zh-hk' => "中文（香港）\xE2\x80\x8E", # Chinese (Hong Kong)
-		'zh-min-nan' => 'Bân-lâm-gú', # Min-nan -- (see bug 8217)
+		'zh-min-nan' => 'Bân-lâm-gú', # Min-nan -- (see T10217)
 		'zh-mo' => "中文（澳門）\xE2\x80\x8E", # Chinese (Macau)
 		'zh-my' => "中文（马来西亚）\xE2\x80\x8E", # Chinese (Malaysia)
 		'zh-sg' => "中文（新加坡）\xE2\x80\x8E", # Chinese (Singapore)
 		'zh-tw' => "中文（台灣）\xE2\x80\x8E", # Chinese (Taiwan)
-		'zh-yue' => '粵語', # Cantonese -- (see bug 8217)
+		'zh-yue' => '粵語', # Cantonese -- (see T10217)
 		'zu' => 'isiZulu' # Zulu
 	];
 }
