@@ -105,7 +105,7 @@ $namespaceAliases = [];
  * Example:
  * @code
  * $namespaceGenderAliases = [
- * 	NS_USER => [ 'male' => 'Male_user', 'female' => 'Female_user' ],
+ *   NS_USER => [ 'male' => 'Male_user', 'female' => 'Female_user' ],
  * ];
  * @endcode
  */
@@ -187,7 +187,7 @@ $dateFormats = [
 $bookstoreList = [
 	'AddALL' => 'http://www.addall.com/New/Partner.cgi?query=$1&type=ISBN',
 	'Barnes & Noble' => 'http://search.barnesandnoble.com/bookSearch/isbnInquiry.asp?isbn=$1',
-	'Amazon.com' => 'http://www.amazon.com/gp/search/?field-isbn=$1'
+	'Amazon.com' => 'https://www.amazon.com/gp/search/?field-isbn=$1'
 ];
 
 /**
@@ -349,6 +349,7 @@ $magicWords = [
 	'directionmark'           => [ 1, 'DIRECTIONMARK', 'DIRMARK' ],
 	'language'                => [ 0, '#LANGUAGE:' ],
 	'contentlanguage'         => [ 1, 'CONTENTLANGUAGE', 'CONTENTLANG' ],
+	'pagelanguage'            => [ 1, 'PAGELANGUAGE' ],
 	'pagesinnamespace'        => [ 1, 'PAGESINNAMESPACE:', 'PAGESINNS:' ],
 	'numberofadmins'          => [ 1, 'NUMBEROFADMINS' ],
 	'formatnum'               => [ 0, 'FORMATNUM' ],
@@ -399,6 +400,7 @@ $specialPageAliases = [
 	'ApiHelp'                   => [ 'ApiHelp' ],
 	'ApiSandbox'                => [ 'ApiSandbox' ],
 	'Ancientpages'              => [ 'AncientPages' ],
+	'AutoblockList'             => [ 'AutoblockList', 'ListAutoblocks' ],
 	'Badtitle'                  => [ 'Badtitle' ],
 	'Blankpage'                 => [ 'BlankPage' ],
 	'Block'                     => [ 'Block', 'BlockIP', 'BlockUser' ],
