@@ -10,14 +10,14 @@
 
 $fallback = 'ko';
 
-$datePreferences = array(
+$datePreferences = [
 	'default',
 	'juche',
 	'juche bracket',
 	'ISO 8601',
-);
+];
 $defaultDateFormat = 'ko';
-$dateFormats = array(
+$dateFormats = [
 	'ko time'            => 'H:i',
 	'ko date'            => 'Y년 M월 j일 (D)',
 	'ko both'            => 'Y년 M월 j일 (D) H:i',
@@ -29,4 +29,4 @@ $dateFormats = array(
 	'juche bracket time' => 'H:i',
 	'juche bracket date' => '주체xoY년 (Y년) M월 j일 (D)',
 	'juche bracket both' => '주체xoY년 (Y년) M월 j일 (D) H:i',
-);
+];
