@@ -32,6 +32,8 @@
 
 require_once __DIR__ . '/Maintenance.php';
 
+use Wikimedia\Rdbms\IMaintainableDatabase;
+
 /**
  * Maintenance script to update image metadata records.
  *
@@ -40,7 +42,7 @@ require_once __DIR__ . '/Maintenance.php';
 class ImageBuilder extends Maintenance {
 
 	/**
-	 * @var Database
+	 * @var IMaintainableDatabase
 	 */
 	protected $dbw;
 
@@ -61,7 +63,8 @@ class ImageBuilder extends Maintenance {
 		$this->dbw = $this->getDB( DB_MASTER );
 		$this->dryrun = $this->hasOption( 'dry-run' );
 		if ( $this->dryrun ) {
-			$GLOBALS['wgReadOnly'] = 'Dry run mode, image upgrades are suppressed';
+			MediaWiki\MediaWikiServices::getInstance()->getReadOnlyMode()
+				->setReason( 'Dry run mode, image upgrades are suppressed' );
 		}
 
 		if ( $this->hasOption( 'missing' ) ) {
