@@ -1,4 +1,4 @@
-/*global moment */
+/* global moment, Uint8Array */
 ( function ( $, mw ) {
 
 	/**
@@ -18,6 +18,9 @@
 	 * @class mw.ForeignStructuredUpload.BookletLayout
 	 * @uses mw.ForeignStructuredUpload
 	 * @extends mw.Upload.BookletLayout
+	 *
+	 * @constructor
+	 * @param {Object} config Configuration options
 	 * @cfg {string} [target] Used to choose the target repository.
 	 *     If nothing is passed, the {@link mw.ForeignUpload#property-target default} is used.
 	 */
@@ -43,11 +46,11 @@
 		return mw.ForeignStructuredUpload.BookletLayout.parent.prototype.initialize.call( this ).then(
 			function () {
 				return $.when(
-					// Point the CategorySelector to the right wiki
+					// Point the CategoryMultiselectWidget to the right wiki
 					booklet.upload.getApi().then( function ( api ) {
 						// If this is a ForeignApi, it will have a apiUrl, otherwise we don't need to do anything
 						if ( api.apiUrl ) {
-							// Can't reuse the same object, CategorySelector calls #abort on its mw.Api instance
+							// Can't reuse the same object, CategoryMultiselectWidget calls #abort on its mw.Api instance
 							booklet.categoriesWidget.api = new mw.ForeignApi( api.apiUrl );
 						}
 						return $.Deferred().resolve();
@@ -104,6 +107,9 @@
 									e.stopPropagation();
 								} );
 						} );
+					}, function ( errorMsg ) {
+						booklet.getPage( 'upload' ).$element.msg( errorMsg );
+						return $.Deferred().resolve();
 					} )
 				);
 			}
@@ -122,7 +128,14 @@
 	 * @return {mw.Upload}
 	 */
 	mw.ForeignStructuredUpload.BookletLayout.prototype.createUpload = function () {
-		return new mw.ForeignStructuredUpload( this.target );
+		return new mw.ForeignStructuredUpload( this.target, {
+			parameters: {
+				errorformat: 'html',
+				errorlang: mw.config.get( 'wgUserLanguage' ),
+				errorsuselocal: 1,
+				formatversion: 2
+			}
+		} );
 	};
 
 	/* Form renderers */
@@ -229,7 +242,7 @@
 			multiline: true,
 			autosize: true
 		} );
-		this.categoriesWidget = new mw.widgets.CategorySelector( {
+		this.categoriesWidget = new mw.widgets.CategoryMultiselectWidget( {
 			// Can't be done here because we don't know the target wiki yet... done in #initialize.
 			// api: new mw.ForeignApi( ... ),
 			$overlay: this.$overlay
@@ -395,7 +408,7 @@
 				}
 
 				try {
-					metadata = mw.libs.jpegmeta( this.result, file.name );
+					metadata = mw.libs.jpegmeta( fileStr, file.name );
 				} catch ( e ) {
 					metadata = null;
 				}
