@@ -1,9 +1,5 @@
-/* jshint devel: true */
 ( function ( mw, $ ) {
 	/**
-	 * @method confirmCloseWindow
-	 * @member mw
-	 *
 	 * Prevent the closing of a window with a confirm message (the onbeforeunload event seems to
 	 * work in most browsers.)
 	 *
@@ -25,6 +21,8 @@
 	 *         // do whatever you wanted to do
 	 *     }
 	 *
+	 * @method confirmCloseWindow
+	 * @member mw
 	 * @param {Object} [options]
 	 * @param {string} [options.namespace] Namespace for the event registration
 	 * @param {string} [options.message]
@@ -109,4 +107,4 @@
 			}
 		};
 	};
-} )( mediaWiki, jQuery );
+}( mediaWiki, jQuery ) );
