@@ -1,32 +1,40 @@
-/*jshint node:true */
+/* eslint-env node */
 module.exports = function ( grunt ) {
-	grunt.loadNpmTasks( 'grunt-contrib-jshint' );
-	grunt.loadNpmTasks( 'grunt-jsonlint' );
-	grunt.loadNpmTasks( 'grunt-banana-checker' );
-	grunt.loadNpmTasks( 'grunt-jscs' );
 	var conf = grunt.file.readJSON( 'skin.json' );
+	grunt.loadNpmTasks( 'grunt-banana-checker' );
+	grunt.loadNpmTasks( 'grunt-eslint' );
+	grunt.loadNpmTasks( 'grunt-jsonlint' );
+	grunt.loadNpmTasks( 'grunt-stylelint' );
 
 	grunt.initConfig( {
-		jshint: {
-			options: {
-				jshintrc: true
-			},
+		eslint: {
 			all: [
-				'*.js'
+				'*.js',
+				'**/*.js',
+				'!node_modules/**'
 			]
 		},
-		jscs: {
-			src: '<%= jshint.all %>'
-		},
-		banana: conf.MessagesDirs,
 		jsonlint: {
 			all: [
+				'*.json',
 				'**/*.json',
 				'!node_modules/**'
 			]
+		},
+		banana: conf.MessagesDirs,
+		stylelint: {
+			options: {
+				syntax: 'less'
+			},
+			all: [
+				'*.{less,css}',
+				'**/*.{less,css}',
+				'!node_modules/**',
+				'!skinStyles/jquery.ui/**'
+			]
 		}
 	} );
 
-	grunt.registerTask( 'test', [ 'jshint', 'jscs', 'jsonlint', 'banana' ] );
+	grunt.registerTask( 'test', [ 'eslint', 'jsonlint', 'banana', 'stylelint' ] );
 	grunt.registerTask( 'default', 'test' );
 };
