@@ -53,7 +53,7 @@ class EditPageTest extends MediaWikiLangTestCase {
 				false
 			],
 			[
-				"An initial section with a fake heder (bug 32617)\n\n== Test == ??\nwtf",
+				"An initial section with a fake heder (T34617)\n\n== Test == ??\nwtf",
 				false
 			],
 			[
@@ -61,7 +61,7 @@ class EditPageTest extends MediaWikiLangTestCase {
 				"Section"
 			],
 			[
-				"== Section== \t\r\n followed by whitespace (bug 35051)",
+				"== Section== \t\r\n followed by whitespace (T37051)",
 				'Section',
 			],
 		];
