@@ -18,9 +18,6 @@ class WfThumbIsStandardTest extends MediaWikiTestCase {
 				[ 300, 225 ],
 				[ 800, 600 ],
 			],
-			'wgMediaHandlers' => [
-				'unknown/unknown' => 'MockBitmapHandler',
-			],
 		] );
 	}
 
@@ -95,6 +92,7 @@ class WfThumbIsStandardTest extends MediaWikiTestCase {
 	 * @dataProvider provideThumbParams
 	 */
 	public function testIsStandard( $message, $expected, $params ) {
+		$this->setService( 'MediaHandlerFactory', new MockMediaHandlerFactory() );
 		$this->assertSame(
 			$expected,
 			wfThumbIsStandard( new FakeDimensionFile( [ 2000, 1800 ] ), $params ),
