@@ -2,6 +2,9 @@
 
 namespace MediaWiki\Auth;
 
+use MediaWiki\MediaWikiServices;
+use Wikimedia\TestingAccessWrapper;
+
 /**
  * @group AuthManager
  * @group Database
@@ -28,7 +31,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiTestCase
 		}
 		$config = new \MultiConfig( [
 			$this->config,
-			\ConfigFactory::getDefaultInstance()->makeConfig( 'main' )
+			MediaWikiServices::getInstance()->getMainConfig()
 		] );
 
 		if ( !$this->manager ) {
@@ -36,11 +39,10 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiTestCase
 		}
 		$this->validity = \Status::newGood();
 
-		$provider = $this->getMock(
-			LocalPasswordPrimaryAuthenticationProvider::class,
-			[ 'checkPasswordValidity' ],
-			[ [ 'loginOnly' => $loginOnly ] ]
-		);
+		$provider = $this->getMockBuilder( LocalPasswordPrimaryAuthenticationProvider::class )
+			->setMethods( [ 'checkPasswordValidity' ] )
+			->setConstructorArgs( [ [ 'loginOnly' => $loginOnly ] ] )
+			->getMock();
 		$provider->expects( $this->any() )->method( 'checkPasswordValidity' )
 			->will( $this->returnCallback( function () {
 				return $this->validity;
@@ -130,7 +132,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiTestCase
 		$provider->setConfig( $this->config );
 		$provider->setLogger( new \Psr\Log\NullLogger() );
 		$provider->setManager( $this->manager );
-		$providerPriv = \TestingAccessWrapper::newFromObject( $provider );
+		$providerPriv = TestingAccessWrapper::newFromObject( $provider );
 
 		$user = $this->getMutableTestUser()->getUser();
 		$userName = $user->getName();
@@ -328,7 +330,6 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiTestCase
 			AuthenticationResponse::newPass( $userName ),
 			$provider->beginPrimaryAuthentication( $reqs )
 		);
-
 	}
 
 	/**
@@ -347,7 +348,7 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiTestCase
 		} elseif ( $type === PasswordDomainAuthenticationRequest::class ) {
 			$req = new $type( [] );
 		} else {
-			$req = $this->getMock( $type );
+			$req = $this->createMock( $type );
 		}
 		$req->action = AuthManager::ACTION_CHANGE;
 		$req->username = $user;
@@ -443,14 +444,14 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiTestCase
 		if ( $type === PasswordAuthenticationRequest::class ) {
 			$changeReq = new $type();
 		} else {
-			$changeReq = $this->getMock( $type );
+			$changeReq = $this->createMock( $type );
 		}
 		$changeReq->action = AuthManager::ACTION_CHANGE;
 		$changeReq->username = $user;
 		$changeReq->password = $newpass;
 		$provider->providerChangeAuthenticationData( $changeReq );
 
-		if ( $loginOnly ) {
+		if ( $loginOnly && $changed ) {
 			$old = 'fail';
 			$new = 'fail';
 			$expectExpiry = null;
@@ -645,7 +646,6 @@ class LocalPasswordPrimaryAuthenticationProviderTest extends \MediaWikiTestCase
 		$this->assertNull( $provider->finishAccountCreation( $user, $user, $res2 ) );
 		$ret = $provider->beginPrimaryAuthentication( $reqs );
 		$this->assertEquals( AuthenticationResponse::PASS, $ret->status, 'new password is set' );
-
 	}
 
 }
