@@ -1,5 +1,7 @@
 <?php
 
+use Wikimedia\Rdbms\LikeMatch;
+
 /**
  * Test the abstract database layer
  * This is a non DBMS depending test.
@@ -658,19 +660,31 @@ class DatabaseSQLTest extends MediaWikiTestCase {
 		return [
 			[
 				'text',
-				"LIKE 'text'"
+				"LIKE 'text' ESCAPE '`'"
 			],
 			[
 				[ 'text', new LikeMatch( '%' ) ],
-				"LIKE 'text%'"
+				"LIKE 'text%' ESCAPE '`'"
 			],
 			[
 				[ 'text', new LikeMatch( '%' ), 'text2' ],
-				"LIKE 'text%text2'"
+				"LIKE 'text%text2' ESCAPE '`'"
 			],
 			[
 				[ 'text', new LikeMatch( '_' ) ],
-				"LIKE 'text_'"
+				"LIKE 'text_' ESCAPE '`'"
+			],
+			[
+				'more_text',
+				"LIKE 'more`_text' ESCAPE '`'"
+			],
+			[
+				[ 'C:\\Windows\\', new LikeMatch( '%' ) ],
+				"LIKE 'C:\\Windows\\%' ESCAPE '`'"
+			],
+			[
+				[ 'accent`_test`', new LikeMatch( '%' ) ],
+				"LIKE 'accent```_test``%' ESCAPE '`'"
 			],
 		];
 	}
