@@ -1,5 +1,8 @@
 <?php
 
+use Wikimedia\Rdbms\TransactionProfiler;
+use Wikimedia\Rdbms\DatabaseDomain;
+
 /**
  * Helper for testing the methods from the Database class
  * @since 1.22
