@@ -20,6 +20,9 @@
  * @file
  */
 
+/**
+ * @group Database
+ */
 class UserPasswordPolicyTest extends MediaWikiTestCase {
 
 	protected $policies = [
@@ -61,6 +64,7 @@ class UserPasswordPolicyTest extends MediaWikiTestCase {
 		$upp = $this->getUserPasswordPolicy();
 
 		$user = User::newFromName( 'TestUserPolicy' );
+		$user->addToDatabase();
 		$user->addGroup( 'sysop' );
 
 		$this->assertArrayEquals(
@@ -106,6 +110,7 @@ class UserPasswordPolicyTest extends MediaWikiTestCase {
 		$upp = $this->getUserPasswordPolicy();
 
 		$user = User::newFromName( $username );
+		$user->addToDatabase();
 		foreach ( $groups as $group ) {
 			$user->addGroup( $group );
 		}
