@@ -4,7 +4,7 @@ namespace MediaWiki\Session;
 
 use Psr\Log\LogLevel;
 use MediaWikiTestCase;
-use User;
+use Wikimedia\TestingAccessWrapper;
 
 /**
  * @group Session
@@ -128,7 +128,7 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 		$provider = new BotPasswordSessionProvider( [
 			'priority' => 40
 		] );
-		$priv = \TestingAccessWrapper::newFromObject( $provider );
+		$priv = TestingAccessWrapper::newFromObject( $provider );
 		$this->assertSame( 40, $priv->priority );
 		$this->assertSame( '_BPsession', $priv->sessionCookieName );
 		$this->assertSame( [], $priv->sessionCookieOptions );
@@ -137,7 +137,7 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 			'priority' => 40,
 			'sessionCookieName' => null,
 		] );
-		$priv = \TestingAccessWrapper::newFromObject( $provider );
+		$priv = TestingAccessWrapper::newFromObject( $provider );
 		$this->assertSame( '_BPsession', $priv->sessionCookieName );
 
 		$provider = new BotPasswordSessionProvider( [
@@ -145,7 +145,7 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 			'sessionCookieName' => 'Foo',
 			'sessionCookieOptions' => [ 'Bar' ],
 		] );
-		$priv = \TestingAccessWrapper::newFromObject( $provider );
+		$priv = TestingAccessWrapper::newFromObject( $provider );
 		$this->assertSame( 'Foo', $priv->sessionCookieName );
 		$this->assertSame( [ 'Bar' ], $priv->sessionCookieOptions );
 	}
@@ -184,7 +184,8 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 	public function testNewSessionInfoForRequest() {
 		$provider = $this->getProvider();
 		$user = static::getTestSysop()->getUser();
-		$request = $this->getMock( 'FauxRequest', [ 'getIP' ] );
+		$request = $this->getMockBuilder( 'FauxRequest' )
+			->setMethods( [ 'getIP' ] )->getMock();
 		$request->expects( $this->any() )->method( 'getIP' )
 			->will( $this->returnValue( '127.0.0.1' ) );
 		$bp = \BotPassword::newFromUser( $user, 'BotPasswordSessionProvider' );
@@ -211,7 +212,8 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 		$provider->setLogger( $logger );
 
 		$user = static::getTestSysop()->getUser();
-		$request = $this->getMock( 'FauxRequest', [ 'getIP' ] );
+		$request = $this->getMockBuilder( 'FauxRequest' )
+			->setMethods( [ 'getIP' ] )->getMock();
 		$request->expects( $this->any() )->method( 'getIP' )
 			->will( $this->returnValue( '127.0.0.1' ) );
 		$bp = \BotPassword::newFromUser( $user, 'BotPasswordSessionProvider' );
@@ -262,7 +264,8 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 		], $logger->getBuffer() );
 		$logger->clearBuffer();
 
-		$request2 = $this->getMock( 'FauxRequest', [ 'getIP' ] );
+		$request2 = $this->getMockBuilder( 'FauxRequest' )
+			->setMethods( [ 'getIP' ] )->getMock();
 		$request2->expects( $this->any() )->method( 'getIP' )
 			->will( $this->returnValue( '10.0.0.1' ) );
 		$data['metadata'] = $dataMD;
@@ -287,7 +290,7 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 		$provider->setLogger( $logger );
 
 		$backend = TestUtils::getDummySessionBackend();
-		$backendPriv = \TestingAccessWrapper::newFromObject( $backend );
+		$backendPriv = TestingAccessWrapper::newFromObject( $backend );
 
 		try {
 			$provider->getAllowedUserRights( $backend );
