@@ -15,32 +15,32 @@
 		}
 	} ) );
 
-	QUnit.test( 'options', 1, function ( assert ) {
+	QUnit.test( 'options', function ( assert ) {
 		assert.ok( mw.user.options instanceof mw.Map, 'options instance of mw.Map' );
 	} );
 
-	QUnit.test( 'user status', 7, function ( assert ) {
-
+	QUnit.test( 'getters (anonymous)', function ( assert ) {
 		// Forge an anonymous user
 		mw.config.set( 'wgUserName', null );
-		delete mw.config.values.wgUserId;
+		mw.config.set( 'wgUserId', null );
 
-		assert.strictEqual( mw.user.getName(), null, 'user.getName() returns null when anonymous' );
-		assert.assertTrue( mw.user.isAnon(), 'user.isAnon() returns true when anonymous' );
-		assert.strictEqual( mw.user.getId(), 0, 'user.getId() returns 0 when anonymous' );
+		assert.strictEqual( mw.user.getName(), null, 'getName()' );
+		assert.strictEqual( mw.user.isAnon(), true, 'isAnon()' );
+		assert.strictEqual( mw.user.getId(), 0, 'getId()' );
+	} );
 
-		// Not part of startUp module
+	QUnit.test( 'getters (logged-in)', function ( assert ) {
 		mw.config.set( 'wgUserName', 'John' );
 		mw.config.set( 'wgUserId', 123 );
 
-		assert.equal( mw.user.getName(), 'John', 'user.getName() returns username when logged-in' );
-		assert.assertFalse( mw.user.isAnon(), 'user.isAnon() returns false when logged-in' );
-		assert.strictEqual( mw.user.getId(), 123, 'user.getId() returns correct ID when logged-in' );
+		assert.equal( mw.user.getName(), 'John', 'getName()' );
+		assert.strictEqual( mw.user.isAnon(), false, 'isAnon()' );
+		assert.strictEqual( mw.user.getId(), 123, 'getId()' );
 
-		assert.equal( mw.user.id(), 'John', 'user.id Returns username when logged-in' );
+		assert.equal( mw.user.id(), 'John', 'user.id()' );
 	} );
 
-	QUnit.test( 'getUserInfos', 3, function ( assert ) {
+	QUnit.test( 'getUserInfo', function ( assert ) {
 		mw.config.set( 'wgUserGroups', [ '*', 'user' ] );
 
 		mw.user.getGroups( function ( groups ) {
@@ -64,7 +64,7 @@
 		this.server.respond();
 	} );
 
-	QUnit.test( 'generateRandomSessionId', 4, function ( assert ) {
+	QUnit.test( 'generateRandomSessionId', function ( assert ) {
 		var result, result2;
 
 		result = mw.user.generateRandomSessionId();
@@ -77,7 +77,7 @@
 
 	} );
 
-	QUnit.test( 'generateRandomSessionId (fallback)', 4, function ( assert ) {
+	QUnit.test( 'generateRandomSessionId (fallback)', function ( assert ) {
 		var result, result2;
 
 		// Pretend crypto API is not there to test the Math.random fallback
@@ -95,6 +95,13 @@
 
 		result2 = mw.user.generateRandomSessionId();
 		assert.notEqual( result, result2, 'different when called multiple times' );
+	} );
 
+	QUnit.test( 'sessionId', function ( assert ) {
+		var result = mw.user.sessionId(),
+			result2 = mw.user.sessionId();
+		assert.equal( typeof result, 'string', 'type' );
+		assert.equal( $.trim( result ), result, 'no leading or trailing whitespace' );
+		assert.equal( result2, result, 'retained' );
 	} );
 }( mediaWiki, jQuery ) );
